"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AirconService = void 0;
const device_1 = require("./device");
class AirconService {
    constructor(platform, accessory, device) {
        this.platform = platform;
        this.accessory = accessory;
        this.device = device;
        const Characteristic = platform.Characteristic;
        // Create the HeaterCooler service
        // Implemented characteristics:
        //    Active
        //    CoolingThresholdTemperature
        //    Current Heater-Cooler State
        //    Current Temperature
        //    HeatingThresholdTemperature
        //    LockPhysicalControls
        //    Name
        //    RotationSpeed
        //    SwingMode
        //    Target Heater-Cooler State
        //    TemperatureDisplayUnits
        this.service = accessory.getService(platform.Service.HeaterCooler) ||
            accessory.addService(platform.Service.HeaterCooler, accessory.context.device.name + " H/C");
        this.service.getCharacteristic(Characteristic.Active)
            .onGet(this.getActive.bind(this))
            .onSet(this.setActive.bind(this));
        this.service.getCharacteristic(Characteristic.CoolingThresholdTemperature)
            .setProps({
            minValue: this.device.get.minSetpoint(),
            maxValue: this.device.get.maxSetpoint(),
        })
            .onGet(this.getCoolingThresholdTemperature.bind(this))
            .onSet(this.setCoolingThresholdTemperature.bind(this));
        this.service.getCharacteristic(Characteristic.CurrentTemperature)
            .onGet(this.getCurrentTemperature.bind(this));
        this.service.getCharacteristic(Characteristic.CurrentHeaterCoolerState)
            .onGet(this.getCurrentHeaterCoolerState.bind(this));
        this.service.getCharacteristic(Characteristic.HeatingThresholdTemperature)
            .setProps({
            minValue: this.device.get.minSetpoint(),
            maxValue: this.device.get.maxSetpoint(),
        })
            .onGet(this.getHeatingThresholdTemperature.bind(this))
            .onSet(this.setHeatingThresholdTemperature.bind(this));
        this.service.getCharacteristic(Characteristic.LockPhysicalControls)
            .onGet(this.getLockPhysicalControls.bind(this))
            .onSet(this.setLockPhysicalControls.bind(this));
        this.service.setCharacteristic(Characteristic.Name, "Aircon");
        this.service.getCharacteristic(Characteristic.RotationSpeed)
            .setProps({ minValue: 0, maxValue: 100, minStep: 25 })
            .onGet(this.getRotationSpeed.bind(this))
            .onSet(this.setRotationSpeed.bind(this));
        this.service.getCharacteristic(Characteristic.SwingMode)
            .onGet(this.getSwingMode.bind(this))
            .onSet(this.setSwingMode.bind(this));
        this.service.getCharacteristic(Characteristic.TargetHeaterCoolerState)
            .onGet(this.getTargetHeaterCoolerState.bind(this))
            .onSet(this.setTargetHeaterCoolerState.bind(this));
        this.service.getCharacteristic(Characteristic.TemperatureDisplayUnits)
            .onGet(this.getTemperatureDisplayUnits.bind(this))
            .onSet(this.setTemperatureDisplayUnits.bind(this));
    }
    updateHomeBridgeState() {
        if (!this.device.get.valid())
            return;
        this.syncCharacteristic('Active', this.getActive());
        this.syncCharacteristic('CoolingThresholdTemperature', this.getCoolingThresholdTemperature());
        this.syncCharacteristic('CurrentTemperature', this.getCurrentTemperature());
        this.syncCharacteristic('CurrentHeaterCoolerState', this.getCurrentHeaterCoolerState());
        this.syncCharacteristic('HeatingThresholdTemperature', this.getHeatingThresholdTemperature());
        this.syncCharacteristic('LockPhysicalControls', this.getLockPhysicalControls());
        this.syncCharacteristic('RotationSpeed', this.getRotationSpeed());
        this.syncCharacteristic('SwingMode', this.getSwingMode());
        this.syncCharacteristic('TargetHeaterCoolerState', this.getTargetHeaterCoolerState());
    }
    syncCharacteristic(characteristic, value) {
        if (this.service.getCharacteristic(this.platform.Characteristic[characteristic]).value != value) {
            this.platform.log.debug(`Updating homebridge characteristics HeaterCooler.${characteristic} => ${value}`);
            this.service.getCharacteristic(this.platform.Characteristic[characteristic]).updateValue(value);
        }
    }
    checkValid() {
        if (!this.device.get.valid())
            throw new this.platform.api.hap.HapStatusError(-70402 /* SERVICE_COMMUNICATION_FAILURE */);
    }
    ////////////////////////////////////////////////////////////////////////////////////////////////////////////
    getActive() {
        this.checkValid();
        const active = this.device.get.active();
        const mode = this.device.get.mode();
        return (active && [device_1.MhacModeTypes.AUTO, device_1.MhacModeTypes.COOL, device_1.MhacModeTypes.HEAT].includes(mode)) ?
            this.platform.Characteristic.Active.ACTIVE : this.platform.Characteristic.Active.INACTIVE;
    }
    async setActive(value) {
        const active = value;
        this.platform.log.debug(`Set characteristic HeaterCooler.Active -> ${value}`);
        this.device.set.active(active);
    }
    ////////////////////////////////////////////////////////////////////////////////////////////////////////////
    getCoolingThresholdTemperature() {
        this.checkValid();
        return this.device.get.setpoint();
    }
    async setCoolingThresholdTemperature(value) {
        const setpoint = value;
        this.platform.log.debug(`Set characteristic HeaterCooler.CoolingThresholdTemperature -> ${setpoint}`);
        this.device.set.setpoint(setpoint);
    }
    ////////////////////////////////////////////////////////////////////////////////////////////////////////////
    getCurrentHeaterCoolerState() {
        this.checkValid();
        let currentState;
        const characteristic = this.platform.Characteristic;
        const mode = this.device.get.mode();
        const currentTemperature = this.device.get.currentTemperature();
        const setpoint = this.device.get.setpoint();
        if (mode == device_1.MhacModeTypes.AUTO) {
            if (currentTemperature > setpoint) {
                currentState = characteristic.CurrentHeaterCoolerState.COOLING;
            }
            else {
                currentState = characteristic.CurrentHeaterCoolerState.HEATING;
            }
        }
        else if (mode == device_1.MhacModeTypes.HEAT) {
            if (currentTemperature < setpoint) {
                currentState = characteristic.CurrentHeaterCoolerState.HEATING;
            }
            else {
                currentState = characteristic.CurrentHeaterCoolerState.IDLE;
            }
        }
        else if (mode == device_1.MhacModeTypes.DRY) {
            currentState = characteristic.CurrentHeaterCoolerState.INACTIVE;
        }
        else if (mode == device_1.MhacModeTypes.FAN) {
            currentState = characteristic.CurrentHeaterCoolerState.INACTIVE;
        }
        else { // state.mode == MhacModeTypes.COOL
            if (currentTemperature > setpoint) {
                currentState = characteristic.CurrentHeaterCoolerState.COOLING;
            }
            else {
                currentState = characteristic.CurrentHeaterCoolerState.IDLE;
            }
        }
        return currentState;
    }
    ////////////////////////////////////////////////////////////////////////////////////////////////////////////
    getCurrentTemperature() {
        this.checkValid();
        return this.device.get.currentTemperature();
    }
    ////////////////////////////////////////////////////////////////////////////////////////////////////////////
    getHeatingThresholdTemperature() {
        this.checkValid();
        return this.device.get.setpoint();
    }
    async setHeatingThresholdTemperature(value) {
        const setpoint = value;
        this.platform.log.debug(`Set characteristic HeaterCooler.HeatingThresholdTemperature -> ${setpoint}`);
        this.device.set.setpoint(setpoint);
    }
    ////////////////////////////////////////////////////////////////////////////////////////////////////////////
    getLockPhysicalControls() {
        this.checkValid();
        return this.device.get.locked();
    }
    async setLockPhysicalControls(value) {
        const locked = value;
        this.platform.log.debug(`Set characteristic HeaterCooler.LockPhysicalControls -> ${locked}`);
        this.device.set.locked(locked ? 1 : 0);
    }
    ////////////////////////////////////////////////////////////////////////////////////////////////////////////
    getRotationSpeed() {
        this.checkValid();
        return this.device.get.fanSpeed() * 25;
    }
    async setRotationSpeed(value) {
        const hw_value = Math.max(1, Math.ceil(value / 25));
        this.platform.log.debug(`Set characteristic HeaterCooler.RotationSpeed -> ${hw_value}`);
        this.device.set.fanSpeed(hw_value);
    }
    ////////////////////////////////////////////////////////////////////////////////////////////////////////////
    getSwingMode() {
        this.checkValid();
        return this.device.get.swingMode();
    }
    async setSwingMode(value) {
        const swing = value;
        this.platform.log.debug(`Set characteristic HeaterCooler.SwingMode -> ${swing}`);
        this.device.set.swingMode(swing);
    }
    ////////////////////////////////////////////////////////////////////////////////////////////////////////////
    getTargetHeaterCoolerState() {
        this.checkValid();
        const characteristic = this.platform.Characteristic;
        switch (this.device.get.mode()) {
            case device_1.MhacModeTypes.AUTO:
                return characteristic.TargetHeaterCoolerState.AUTO;
            case device_1.MhacModeTypes.HEAT:
                return characteristic.TargetHeaterCoolerState.HEAT;
            case device_1.MhacModeTypes.DRY:
                return characteristic.TargetHeaterCoolerState.AUTO;
            case device_1.MhacModeTypes.FAN:
                return characteristic.TargetHeaterCoolerState.AUTO;
            case device_1.MhacModeTypes.COOL:
            default:
                return characteristic.TargetHeaterCoolerState.COOL;
        }
    }
    async setTargetHeaterCoolerState(value) {
        let mode = null;
        switch (value) {
            case this.platform.Characteristic.TargetHeaterCoolerState.AUTO:
                mode = device_1.MhacModeTypes.AUTO;
                break;
            case this.platform.Characteristic.TargetHeaterCoolerState.COOL:
                mode = device_1.MhacModeTypes.COOL;
                break;
            case this.platform.Characteristic.TargetHeaterCoolerState.HEAT:
                mode = device_1.MhacModeTypes.HEAT;
                break;
        }
        if (mode != null) {
            this.platform.log.debug(`Set characteristic HeaterCooler.TargetHeaterCoolerState -> ${value}`);
            this.device.set.mode(mode);
        }
    }
    ////////////////////////////////////////////////////////////////////////////////////////////////////////////
    getTemperatureDisplayUnits() {
        this.checkValid();
        let units = this.accessory.context.device.units;
        units = units ? this.platform.Characteristic.TemperatureDisplayUnits.FAHRENHEIT :
            this.platform.Characteristic.TemperatureDisplayUnits.CELSIUS;
        return units;
    }
    async setTemperatureDisplayUnits(value) {
        this.accessory.context.device.units = value;
    }
}
exports.AirconService = AirconService;
//# sourceMappingURL=aircon.js.map