"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DehumidifierService = void 0;
const device_1 = require("./device");
class DehumidifierService {
    constructor(platform, accessory, device) {
        this.platform = platform;
        this.device = device;
        const Characteristic = platform.Characteristic;
        // Create the dehumidifier service
        // Implemented characteristics:
        //    Active
        //    Current Humidifier-Dehumidifier State
        //    Name
        //    RotationSpeed
        //    SwingMode
        //    Target Humidifier-Dehumidifier State
        this.service = accessory.getService(platform.Service.HumidifierDehumidifier) ||
            accessory.addService(platform.Service.HumidifierDehumidifier, accessory.context.device.name + " Dehumidifier");
        this.service.getCharacteristic(Characteristic.Active)
            .onGet(this.getActive.bind(this))
            .onSet(this.setActive.bind(this));
        this.service.getCharacteristic(Characteristic.CurrentHumidifierDehumidifierState)
            .onGet(this.getCurrentHumidifierDehumidifierState.bind(this));
        this.service.setCharacteristic(Characteristic.Name, "Dehumidifier");
        this.service.getCharacteristic(Characteristic.RelativeHumidityDehumidifierThreshold)
            .onGet(this.getRelativeHumidityDehumidifierThreshold.bind(this));
        this.service.getCharacteristic(Characteristic.RotationSpeed)
            .setProps({ minValue: 0, maxValue: 100, minStep: 25 })
            .onGet(this.getRotationSpeed.bind(this))
            .onSet(this.setRotationSpeed.bind(this));
        this.service.getCharacteristic(Characteristic.SwingMode)
            .onGet(this.getSwingMode.bind(this))
            .onSet(this.setSwingMode.bind(this));
        this.service.getCharacteristic(Characteristic.TargetHumidifierDehumidifierState)
            .setProps({
            minValue: Characteristic.TargetHumidifierDehumidifierState.DEHUMIDIFIER,
            maxValue: Characteristic.TargetHumidifierDehumidifierState.DEHUMIDIFIER,
            validValues: [Characteristic.TargetHumidifierDehumidifierState.DEHUMIDIFIER]
        })
            .onGet(this.getTargetHumidifierDehumidifierState.bind(this))
            .onSet(this.setTargetHumidifierDehumidifierState.bind(this));
    }
    updateHomeBridgeState() {
        if (!this.device.get.valid())
            return;
        this.syncCharacteristic('Active', this.getActive());
        this.syncCharacteristic('CurrentHumidifierDehumidifierState', this.getCurrentHumidifierDehumidifierState());
        this.syncCharacteristic('RotationSpeed', this.getRotationSpeed());
        this.syncCharacteristic('SwingMode', this.getSwingMode());
        this.syncCharacteristic('TargetHumidifierDehumidifierState', this.getTargetHumidifierDehumidifierState());
    }
    syncCharacteristic(characteristic, value) {
        if (this.service.getCharacteristic(this.platform.Characteristic[characteristic]).value != value) {
            this.platform.log.debug(`Updating homebridge characteristics Dehumidifier.${characteristic} => ${value}`);
            this.service.getCharacteristic(this.platform.Characteristic[characteristic]).updateValue(value);
        }
    }
    checkValid() {
        if (!this.device.get.valid())
            throw new this.platform.api.hap.HapStatusError(-70402 /* SERVICE_COMMUNICATION_FAILURE */);
    }
    ////////////////////////////////////////////////////////////////////////////////////////////////////////////
    getActive() {
        this.checkValid();
        const active = this.device.get.active();
        const mode = this.device.get.mode();
        return active && mode == device_1.MhacModeTypes.DRY ? this.platform.Characteristic.Active.ACTIVE : this.platform.Characteristic.Active.INACTIVE;
    }
    async setActive(value) {
        const active = value;
        this.platform.log.debug(`Set characteristic HumidifierDehumidifier.Active -> ${value}`);
        if (active) {
            this.device.set.mode(device_1.MhacModeTypes.DRY);
        }
        this.device.set.active(active);
    }
    ////////////////////////////////////////////////////////////////////////////////////////////////////////////
    getCurrentHumidifierDehumidifierState() {
        this.checkValid();
        const active = this.device.get.active();
        const mode = this.device.get.mode();
        if (active && mode == device_1.MhacModeTypes.DRY) {
            return this.platform.Characteristic.CurrentHumidifierDehumidifierState.DEHUMIDIFYING;
        }
        else {
            return this.platform.Characteristic.CurrentHumidifierDehumidifierState.INACTIVE;
        }
    }
    ////////////////////////////////////////////////////////////////////////////////////////////////////////////
    getRelativeHumidityDehumidifierThreshold() {
        this.checkValid();
        const active = this.device.get.active();
        const mode = this.device.get.mode();
        return (active && mode == device_1.MhacModeTypes.DRY) ? 50 : 0;
    }
    ////////////////////////////////////////////////////////////////////////////////////////////////////////////
    getRotationSpeed() {
        this.checkValid();
        return this.device.get.fanSpeed() * 25;
    }
    async setRotationSpeed(value) {
        const hw_value = Math.ceil(value / 25);
        this.platform.log.debug(`Set characteristic HumidifierDehumidifier.RotationSpeed -> ${hw_value}`);
        this.device.set.fanSpeed(hw_value);
    }
    ////////////////////////////////////////////////////////////////////////////////////////////////////////////
    getSwingMode() {
        this.checkValid();
        return this.device.get.swingMode();
    }
    async setSwingMode(value) {
        const swing = value;
        this.platform.log.debug(`Set characteristic HumidifierDehumidifier.SwingMode -> ${swing}`);
        this.device.set.swingMode(swing);
    }
    ////////////////////////////////////////////////////////////////////////////////////////////////////////////
    getTargetHumidifierDehumidifierState() {
        this.checkValid();
        return this.platform.Characteristic.TargetHumidifierDehumidifierState.DEHUMIDIFIER;
    }
    setTargetHumidifierDehumidifierState(value) {
        if (value != this.platform.Characteristic.TargetHumidifierDehumidifierState.DEHUMIDIFIER) {
            this.platform.log.error(`Invalid state for TargetHumidifierDehumidifierState => ${value}`);
        }
    }
}
exports.DehumidifierService = DehumidifierService;
//# sourceMappingURL=dehumidifier.js.map