/// <reference types="node" />
import { EventEmitter } from 'events';
import { Logger } from 'homebridge';
export declare enum MhacModeTypes {
    AUTO = 0,
    HEAT = 1,
    DRY = 2,
    FAN = 3,
    COOL = 4
}
export declare const EVENT_CHANGED = "changed";
export declare const EVENT_UPDATED = "updated";
/**
 * Hardware interface class for the MH-AC-WIFI-1
 *
 * This class provides local HTTP access to the Intesis WIFI control board used
 * in the Mitsubishi Heavy Aircon.
 *
 * If enabled (via the `startSynchronization` method), the object periodically
 * polls all of the device's sensors and reflects those back in the `state`
 * property of the class.  When sensors change values, the object will emit
 * an `update` event to notify listeners that at least one value in the state
 * has changed.  For specific changes, listeners can monitor the `changed`
 * event for the specific state that changed.
 *
 * The aircon status should be obtained through the `get` API such as obj.get.active()
 * or obj.get.currentTemperature().  To control the aircon, you use the `set`
 * API such as obj.set.active(1) or object.setFanSpeed(2).
 */
export declare class MHACWIFI1 extends EventEmitter {
    private log;
    private host;
    private username;
    private password;
    private slowThreshold;
    private minSetpoint;
    private maxSetpoint;
    private syncPeriod;
    private sessionID;
    private syncTimeout;
    private sensorMap;
    private previousState;
    private state;
    constructor(log: Logger, host: string, // IP or hostname
    username: string, // Username used for authentication
    password: string, // Password used for authentication
    slowThreshold?: number, // Number of milliseconds before reporting a slow connection
    minSetpoint?: number, // Minimum value for the setpoint temperature
    maxSetpoint?: number, // Maximum value for the setpoint temperature
    syncPeriod?: number);
    /**
     * Public API for getting state values
     */
    get: {
        active: () => number;
        currentTemperature: () => number;
        fanSpeed: () => number;
        locked: () => number;
        maxSetpoint: () => number;
        minSetpoint: () => number;
        mode: () => number;
        outdoorTemperature: () => number;
        setpoint: () => number;
        swingMode: () => number;
        valid: () => boolean;
    };
    /**
     * Public API for setting state values
     *
     */
    set: {
        active: (value: number) => Promise<void>;
        fanSpeed: (value: number) => Promise<void>;
        locked: (value: number) => Promise<void>;
        maxSetpoint: (value: number) => Promise<void>;
        minSetpoint: (value: number) => Promise<void>;
        mode: (value: number) => Promise<void>;
        setpoint: (value: number) => Promise<void>;
        swingMode: (value: number) => Promise<void>;
    };
    /**
     * Enables periodic timer for polling all device sensor states
     */
    startSynchronization(): void;
    /**
     * Stops the periodic polling for sensor states
     */
    stopSynchronization(): void;
    /**
     * Requests hardware configuration information from the device
     *
     * @returns Object containing device information such as firmware version
     */
    getInfo(): Promise<Record<string, string>>;
    /**
     * Performs login with the device
     *
     * @returns Session ID if login is successful
     */
    login(): Promise<string>;
    /**
     * Performs device logout
     */
    logout(): Promise<void>;
    /**
     * Returns the services that are currently available
     *
     * @returns List of service commands available on device
     */
    getAvailableServices(): Promise<string[]>;
    /**
     * Returns the services that are currently available
     *
     * @returns List of service commands available on device
     */
    getAvailableDatapoints(): Promise<Record<string, unknown>>;
    /**
     * Queries all sensors on the device
     *
     * After the device query, the returned values are normalized into the
     * "state" object variable.
     *
     */
    refreshState(): Promise<void>;
    /**
     * Reads all sensors values from the device and caches them into the `state` variable.
     */
    private syncState;
    /**
     * Clears all state information and sessionID
     */
    private resetState;
    /**
     * Converts the raw sensor data into normalized state values
     *
     * @param states
     */
    private parseState;
    /**
     * Checks previous and current state for differences and emits signal on difference
     *
     * Emits a EVENT_CHANGED event for each changed property with property name, old
     * value, and new value.  Emits a generic EVENT_UPDATED property if any property
     * values have changed.
     */
    private checkForChange;
    /**
     * Sets the given sensor to the given value
     *
     * @param attr  Attribute name
     * @param value Normalized value (will be mapped into device specific value)
     */
    private setState;
    /**
     * Sends an HTTP POST request to the device with the given command
     *
     * This function takes care of adding sessionID credentials to the request
     * from current login.  Returned result is the "data" field in the
     * response json payload.
     *
     * @param command   Command for the request
     * @param data      Parameters associated with the command
     * @returns         JSON data returned by the device
     */
    private httpRequest;
    /**
     * Converts the SensorCOnfigMap into a two-way translation structure for converting
     * uid <-> attrName and human-values <-> machine-values.
     */
    private _buildSensorMap;
}
//# sourceMappingURL=device.d.ts.map